var/_math/Math = new()
_math
	proc

		// Title: Arc Tangent Snippet
		// Credit to: Lummox JR
		// Contributed by: Jtgibson
		// http://www.byond.com/members/Jtgibson/forum?id=8
		Atan2(x, y)
			if(!x && !y) return 0
			var/a = arccos(x / sqrt(x*x + y*y))
			return y >= 0 ? a : -a

		Ceiling(x)
			return -round(-x)

		Clamp(val, min, max)
			return max(min, min(val, max))

		// cotangent
		Cot(x)
			return 1 / Tan(x)

		// cosecant
		Csc(x)
			return 1 / sin(x)

		Default(a, b)
			return a ? a : b

		Floor(x)
			return round(x)

		// Greatest Common Divisor - Euclid's algorithm
		Gcd(a, b)
			return b ? Gcd(b, a % b) : a

		Inverse(x)
			return 1 / x

		IsAboutEqual(a, b, deviation = 0.1)
			return abs(a - b) <= deviation

		IsEven(x)
			return x % 2 == 0

		// Returns true if val is from min to max, inclusive.
		IsInRange(val, min, max)
			return min <= val && val <= max

		IsInteger(x)
			return Floor(x) == x

		IsOdd(x)
			return !IsEven(x)

		// Least Common Multiple
		Lcm(a, b)
			return abs(a) / Gcd(a, b) * abs(b)

		// Performs a linear interpolation between a and b.
		// Note that amount=0 returns a, amount=1 returns b, and
		// amount=0.5 returns the mean of a and b.
		Lerp(a, b, amount = 0.5)
			return a + (b - a) * amount

		Mean(...)
			var
				values 	= 0
				sum		= 0
			for(var/val in args)
				values++
				sum += val
			return sum / values


		// Returns the nth root of x.
		Root(n, x)
			return x ** (1 / n)

		// secant
		Sec(x)
			return 1 / cos(x)

		// The quadratic formula. Returns a list with the solutions, or an empty list
		// if they are imaginary.
		SolveQuadratic(a, b, c)
			ASSERT(a)
			. = list()
			var
				d		= b*b - 4 * a * c
				bottom  = 2 * a
			if(d < 0) return
			var/root = sqrt(d)
			. += (-b + root) / bottom
			if(!d) return
			. += (-b - root) / bottom

		// tangent
		Tan(x)
			return sin(x) / cos(x)

		ToDegrees(radians)
						  // 180 / Pi
			return radians * 57.2957795

		ToRadians(degrees)
						  // Pi / 180
			return degrees * 0.0174532925

		// min is inclusive, max is exclusive
		Wrap(val, min, max)
			var
				d = max - min
				t = Floor((val - min) / d)
			return val - (t * d)

	var/const
		Pi 		= 3.14159265
		E		= 2.71828183
		Sqrt2	= 1.41421356